@@ -21,6 +21,7 @@
 */
 
 #include <QDir>
+#include <QOperatingSystemVersion>
 
 #include "qutecsound.h"
 #include "configdialog.h"
@@ -28,6 +29,7 @@
 #include "types.h"
 #include "configlists.h"
 
+
 typedef QPair<QString, QString> QStringPair;
 
 #ifdef QCS_RTMIDI
@@ -155,6 +157,7 @@ ConfigDialog::ConfigDialog(CsoundQt *parent, Options *options, ConfigLists *conf
 	fontComboBox->setCurrentIndex(fontComboBox->findText(m_options->font) );
 	fontSizeComboBox->setCurrentIndex(fontSizeComboBox->findText(QString::number((int) m_options->fontPointSize)));
 	lineNumbersCheckBox->setChecked(m_options->showLineNumberArea);
+
 	lineEndingComboBox->setCurrentIndex(m_options->lineEnding);
 	consoleFontComboBox->setCurrentIndex(consoleFontComboBox->findText(m_options->consoleFont) );
 	consoleFontSizeComboBox->setCurrentIndex(consoleFontSizeComboBox->findText(QString::number((int) m_options->consoleFontPointSize)));
@@ -170,9 +173,9 @@ ConfigDialog::ConfigDialog(CsoundQt *parent, Options *options, ConfigLists *conf
 	consoleBgColorPushButton->setPalette(palette);
 
 	pixmap.fill(m_options->editorBgColor);
-	editorBgColorButton->setIcon(pixmap);
+    // editorBgColorButton->setIcon(pixmap);
 	palette = QPalette(m_options->editorBgColor);
-	editorBgColorButton->setPalette(palette);
+    // editorBgColorButton->setPalette(palette);
 
 	tabWidthSpinBox->setValue(m_options->tabWidth);
 	tabIndentCheckBox->setChecked(m_options->tabIndents);
@@ -193,6 +196,7 @@ ConfigDialog::ConfigDialog(CsoundQt *parent, Options *options, ConfigLists *conf
 	autoCompleteCheckBox->setChecked(m_options->autoComplete);
 	autoParameterModeCheckBox->setChecked(m_options->autoParameterMode);
 	widgetsCheckBox->setChecked(m_options->enableWidgets);
+	midiCcToCurrentPageOnlyCheckBox->setChecked(m_options->midiCcToCurrentPageOnly);
 	showWidgetsOnRunCheckBox->setChecked(m_options->showWidgetsOnRun);
 	showTooltipsCheckBox->setChecked(m_options->showTooltips);
     graphUpdateRateSpinBox->setValue(m_options->graphUpdateRate);
@@ -204,6 +208,7 @@ ConfigDialog::ConfigDialog(CsoundQt *parent, Options *options, ConfigLists *conf
 	fontScalingSpinBox->setValue(m_options->fontScaling);
 	fontOffsetSpinBox->setValue(m_options->fontOffset);
     tabShortcutActiveCheckBox->setChecked(m_options->tabShortcutActive);
+    highlightScoreCheckBox->setChecked(m_options->highlightScore);
 
 	if (m_options->useAPI)
 		ApiRadioButton->setChecked(true);
@@ -227,8 +232,6 @@ ConfigDialog::ConfigDialog(CsoundQt *parent, Options *options, ConfigLists *conf
 
     checkSyntaxBeforeRunCheckBox->setChecked(m_options->checkSyntaxBeforeRun);
 
-
-
 	BufferSizeLineEdit->setText(QString::number(m_options->bufferSize));
 	BufferSizeCheckBox->setChecked(m_options->bufferSizeActive);
 	BufferSizeLineEdit->setEnabled(m_options->bufferSizeActive);
@@ -283,12 +286,24 @@ ConfigDialog::ConfigDialog(CsoundQt *parent, Options *options, ConfigLists *conf
 
     numChannelsCheckBox->setChecked(m_options->overrideNumChannels);
     numChannelsSpinBox->setValue(m_options->numChannels);
-    numChannelsSpinBox->setMinimum(0);
+    numChannelsSpinBox->setMinimum(1);
     numChannelsSpinBox->setMaximum(64);
 
+    numInputChannelsSpinBox->setValue(m_options->numInputChannels);
+    numInputChannelsSpinBox->setMinimum(1);
+    numInputChannelsSpinBox->setMaximum(64);
+
     realtimeCheckBox->setChecked(m_options->realtimeFlag);
     sampleAccurateCheckBox->setChecked(m_options->sampleAccurateFlag);
 
+    //limiter
+    bool limiterAvailable = csoundGetVersion()>=6160;
+    limiterCheckBox->setEnabled(limiterAvailable);
+    limiterSpinBox->setEnabled(limiterAvailable);
+    if (limiterAvailable) {
+        limiterCheckBox->setChecked(m_options->useLimiter);
+        limiterSpinBox->setValue(m_options->limitValue);
+    }
 
 	JackNameLineEdit->setText(m_options->rtJackName);
 	RtMidiModuleComboBox->setCurrentIndex(RtMidiModuleComboBox->findData(m_options->rtMidiModule));
@@ -321,6 +336,7 @@ ConfigDialog::ConfigDialog(CsoundQt *parent, Options *options, ConfigLists *conf
 	//  defaultCsdLineEdit->setText(m_options->defaultCsd);
 	//  defaultCsdLineEdit->setEnabled(m_options->defaultCsdActive);
 	favoriteLineEdit->setText(m_options->favoriteDir);
+    examplePathlineEdit->setText(m_options->examplePath);
 	pythonDirLineEdit->setText(m_options->pythonDir);
 	pythonExecutableLineEdit->setText(m_options->pythonExecutable);
 	csoundExecutableLineEdit->setText(m_options->csoundExecutable);
@@ -351,7 +367,8 @@ ConfigDialog::ConfigDialog(CsoundQt *parent, Options *options, ConfigLists *conf
 	connect(rawWaveToolButton, SIGNAL(clicked()), this, SLOT(browseRawWaveDir()));
 	//  connect(defaultCsdToolButton, SIGNAL(clicked()), this, SLOT(browseDefaultCsd()));
 	connect(favoriteToolButton, SIGNAL(clicked()), this, SLOT(browseFavorite()));
-	connect(terminalToolButton, SIGNAL(clicked()), this, SLOT(browseTerminal()));
+    connect(examplePathToolButton, SIGNAL(clicked()), this, SLOT(browseExamplePath()) );
+    connect(terminalToolButton, SIGNAL(clicked()), this, SLOT(browseTerminal()));
 	connect(browserToolButton, SIGNAL(clicked()), this, SLOT(browseBrowser()));
 	connect(dotToolButton, SIGNAL(clicked()), this, SLOT(browseDot()));
 	connect(waveEditorToolButton, SIGNAL(clicked()), this, SLOT(browseWaveEditor()));
@@ -369,7 +386,7 @@ ConfigDialog::ConfigDialog(CsoundQt *parent, Options *options, ConfigLists *conf
 	connect(midiOutputToolButton, SIGNAL(released()), this, SLOT(selectMidiOutput()));
 	connect(consoleFontColorPushButton, SIGNAL(released()), this, SLOT(selectTextColor()));
 	connect(consoleBgColorPushButton, SIGNAL(released()), this, SLOT(selectBgColor()));
-	connect(editorBgColorButton, SIGNAL(released()), this, SLOT(selectEditorBgColor()));
+    // connect(editorBgColorButton, SIGNAL(released()), this, SLOT(selectEditorBgColor()));
 
 	connect(clearTemplatePushButton,SIGNAL(released()), this, SLOT(clearTemplate()));
 	connect(defaultTemplatePushButton,SIGNAL(released()), this, SLOT(defaultTemplate()));
@@ -382,7 +399,7 @@ ConfigDialog::ConfigDialog(CsoundQt *parent, Options *options, ConfigLists *conf
     connect(pythonExecutableToolButton,SIGNAL(clicked()),this, SLOT(browsePythonExecutable()));
 
     connect(testAudioSetupButton, SIGNAL(released()), this, SLOT(testAudioSetup()));
-    connect(envirRecommendButton, SIGNAL(clicked()), this, SLOT(recommendEnvironmentSettings()));
+    // connect(envirRecommendButton, SIGNAL(clicked()), this, SLOT(recommendEnvironmentSettings()));
 
 	//connect(RtMidiModuleComboBox, SIGNAL(currentTextChanged(QString)), this, SLOT(checkRtMidiModule(QString)) );
 
@@ -405,6 +422,8 @@ void ConfigDialog::onRtModuleComboBoxChanged(int index) {
     // Todo: remember the settings for each backend in a hash table, and set it to the last
     // known value when changed back. Otherwise set it to the default adc/dac
     auto currentText = this->RtModuleComboBox->currentText();
+	auto currentOperatingSystem = QOperatingSystemVersion::current();
+    //qDebug() << "module: " << currentText << "Op.system type is: " << currentOperatingSystem.type() << currentOperatingSystem.name();
     if(currentText == "null") {
         RtInputLineEdit->setText("");
         RtOutputLineEdit->setText("");
@@ -424,7 +443,11 @@ void ConfigDialog::onRtModuleComboBoxChanged(int index) {
             m_options->useSystemSamplerate = false;
             m_options->samplerate = 0;
         }
-    } else {
+    } else if (currentText.startsWith("portaudio") && currentOperatingSystem.type()==QOperatingSystemVersion::MacOS ) { // on newer Mac's the internal microphone has 1 channel and that causes problems for portaudio. Disable input by default
+			qDebug() << "Set audio input to none for portaudio on MacOS";
+			RtInputLineEdit->setText("");
+			RtOutputLineEdit->setText("dac");
+	} else {
         RtInputLineEdit->setText("adc");
         RtOutputLineEdit->setText("dac");
     }
@@ -480,6 +503,7 @@ void ConfigDialog::accept()
 	m_options->autoComplete = autoCompleteCheckBox->isChecked();
 	m_options->autoParameterMode = autoParameterModeCheckBox->isChecked();
 	m_options->enableWidgets = widgetsCheckBox->isChecked();
+	m_options->midiCcToCurrentPageOnly = midiCcToCurrentPageOnlyCheckBox->isChecked();
 	m_options->showWidgetsOnRun = showWidgetsOnRunCheckBox->isChecked();
 	m_options->showTooltips = showTooltipsCheckBox->isChecked();
 	m_options->enableFLTK = enableFLTKCheckBox->isChecked();
@@ -511,8 +535,10 @@ void ConfigDialog::accept()
 	m_options->HwBufferSize = HwBufferSizeLineEdit->text().toInt();
 	m_options->HwBufferSizeActive = HwBufferSizeCheckBox->isChecked();
     // m_options->dither = DitherCheckBox->isChecked();
-    m_options->realtimeFlag =
+    m_options->realtimeFlag = realtimeCheckBox->isChecked(); // this was empty after = ?
     // m_options->newParser = newParserCheckBox->isChecked() ? 1 : 0;
+    m_options->useLimiter = limiterCheckBox->isEnabled() && limiterCheckBox->isChecked();
+    m_options->limitValue = limiterSpinBox->value();
 	m_options->multicore = multicoreCheckBox->isChecked();
 	m_options->numThreads = numThreadsSpinBox->value();
 	m_options->additionalFlags = AdditionalFlagsLineEdit->text();
@@ -548,9 +574,12 @@ void ConfigDialog::accept()
     }
     m_options->overrideNumChannels = numChannelsCheckBox->isChecked();
     m_options->numChannels = numChannelsSpinBox->value();
+
+    m_options->numInputChannels = numInputChannelsSpinBox->value();
     m_options->realtimeFlag = realtimeCheckBox->isChecked();
     m_options->sampleAccurateFlag = sampleAccurateCheckBox->isChecked();
 
+
 	m_options->rtMidiModule = RtMidiModuleComboBox->currentText();
 	m_options->rtMidiInputDevice = RtMidiInputLineEdit->text();
 	m_options->rtMidiOutputDevice = RtMidiOutputLineEdit->text();
@@ -579,6 +608,7 @@ void ConfigDialog::accept()
 	//  m_options->defaultCsdActive = defaultCsdCheckBox->isChecked();
 	//  m_options->defaultCsd = defaultCsdLineEdit->text();
 	m_options->favoriteDir = favoriteLineEdit->text();
+    m_options->examplePath = examplePathlineEdit->text();
 	m_options->pythonDir = pythonDirLineEdit->text();
 	m_options->pythonExecutable = pythonExecutableLineEdit->text();
 	m_options->csoundExecutable = csoundExecutableLineEdit->text();
@@ -596,6 +626,7 @@ void ConfigDialog::accept()
 
 	m_options->csdTemplate = templateTextEdit->toPlainText();
     m_options->checkSyntaxBeforeRun = checkSyntaxBeforeRunCheckBox->isChecked();
+    m_options->highlightScore = highlightScoreCheckBox->isChecked();
 
 	//  emit(changeFont());
 	QDialog::accept();
@@ -697,6 +728,12 @@ void ConfigDialog::browseFavorite()
 	favoriteLineEdit->setText(m_options->favoriteDir);
 }
 
+void ConfigDialog::browseExamplePath()
+{
+    browseDir(m_options->examplePath);
+    examplePathlineEdit->setText(m_options->examplePath);
+}
+
 void ConfigDialog::browseTerminal()
 {
 	browseFile(m_options->terminal);
@@ -947,9 +984,9 @@ void ConfigDialog::selectEditorBgColor()
 		m_options->editorBgColor = color;
 		QPixmap pixmap (64,64);
 		pixmap.fill(m_options->editorBgColor);
-		editorBgColorButton->setIcon(pixmap);
+        // editorBgColorButton->setIcon(pixmap);
 		QPalette palette(m_options->editorBgColor);
-		editorBgColorButton->setPalette(palette);
+        // editorBgColorButton->setPalette(palette);
 	}
 }
 
