@@ -739,7 +739,12 @@ void DocumentPage::updateCsLadspaText()
 QString DocumentPage::getQml()
 {
     QString qml = m_widgetLayouts[0]->getQml();
-    return qml;
+	return qml;
+}
+
+QString DocumentPage::getMidiControllerInstrument()
+{
+	return m_widgetLayouts[0]->getMidiControllerInstrument();
 }
 
 void DocumentPage::updateCabbageText()
@@ -1183,7 +1188,7 @@ void DocumentPage::queueMidiIn(std::vector< unsigned char > *message)
         qDebug() << "MIDI message ignored. nBytes=" << nBytes << " status =" << (int)message->at(0);
         return;
     }
-    if ( ((d->midiWriteCounter + 1) % QCS_MAX_MIDI_QUEUE) != d->midiReadCounter) {
+	if ( (((d->midiWriteCounter + 1) % QCS_MAX_MIDI_QUEUE) != d->midiReadCounter) && acceptsMidiCC) {
         int index = d->midiWriteCounter;
         for (unsigned int i = 0; i < nBytes; i++) {
             d->midiQueue[index][i] = (int)message->at(i);
@@ -1701,6 +1706,10 @@ void DocumentPage::setHighlightingTheme(QString theme) {
 
 }
 
+void DocumentPage::enableScoreSyntaxHighlighting(bool status) {
+    m_view->enableScoreSyntaxHighlighting(status);
+}
+
 void DocumentPage::setParsedUDOs(QStringList udos) {
     m_view->setParsedUDOs(udos);
 }
