@@ -30,7 +30,7 @@
 #include "texteditor.h"
 
 
-
+static const QStringList tagWords = {"CsInstruments", "CsScore", "CsoundSynthesizer", "CsOptions"};
 
 DocumentView::DocumentView(QWidget * parent, OpEntryParser *opcodeTree) :
 	BaseView(parent,opcodeTree)
@@ -42,10 +42,11 @@ DocumentView::DocumentView(QWidget * parent, OpEntryParser *opcodeTree) :
 	m_hoverText->show();
 	m_hoverText->setStyleSheet(
 				"QLabel {"
-				"border: 1px solid black;"
-//				"border-radius: 4px;"
-				"padding: 2px;"
-				"background-color: #eaeac5;"
+                "  border: 1px solid black;"
+                "  border-radius: 4px;"
+                "  padding: 2px;"
+                "  background-color: #eaeac5;"
+                "  color: #202020; "
 				"};");
 	m_hoverText->setWordWrap(true);
 	m_hoverWidget->hide();
@@ -107,10 +108,10 @@ DocumentView::DocumentView(QWidget * parent, OpEntryParser *opcodeTree) :
 	syntaxMenu = new MySyntaxMenu(m_mainEditor);
 	//  syntaxMenu->setFocusPolicy(Qt::NoFocus);
 	syntaxMenu->setAutoFillBackground(true);
-    QPalette p = syntaxMenu->palette();
-	p.setColor(QPalette::WindowText, Qt::blue);
-	p.setColor(static_cast<QPalette::ColorRole>(9), QColor("#eaeac5")); // was: Qt::yellow
-	syntaxMenu->setPalette(p);
+    // QPalette p = syntaxMenu->palette();
+    // p.setColor(QPalette::WindowText, Qt::blue);
+    // p.setColor(static_cast<QPalette::ColorRole>(9), QColor("#eaeac5")); // was: Qt::yellow
+    // syntaxMenu->setPalette(p);
 	connect(syntaxMenu,SIGNAL(keyPressed(QString)),
 			m_mainEditor, SLOT(insertPlainText(QString)));
 
@@ -160,6 +161,9 @@ void DocumentView::print(QPrinter *printer)
 
 void DocumentView::updateContext()
 {
+    // this code is too expensive to be run at EACH keystroke. We need to think of something better
+    // Until then I am disabling it (EM, Nov. 21)
+    return;
 	QStringList contextStart;
 	contextStart << "<CsInstruments>" << "<CsScore" << "<CsOptions>";
 	QTextCursor cursor = m_mainEditor->textCursor();
@@ -192,7 +196,7 @@ void DocumentView::updateContext()
 			linecursor.movePosition(QTextCursor::EndOfLine);
 		}
 		QString orc = cursor.selection().toPlainText();
-		updateOrcContext(orc);
+        updateOrcContext(orc);
 	}
 }
 
@@ -530,7 +534,7 @@ QString DocumentView::getWidgetsText()
 	// With tags including presets, in new xml format. For text that is being edited in the text editor
 	// Includes presets text
 	qDebug() << "Not implemented and will crash!";
-    return "Not implemeneted.";
+    return "Not implemented.";
 }
 
 int DocumentView::getViewMode()
@@ -849,6 +853,7 @@ void DocumentView::textChanged()
 //			}
 			cursor.select(QTextCursor::WordUnderCursor);
 			QString word = cursor.selectedText();
+            QString wordlow = word.toLower();
 			if (word == ",") {
 				cursor.movePosition(QTextCursor::PreviousCharacter, QTextCursor::MoveAnchor, 2);
 				cursor.movePosition(QTextCursor::StartOfWord);
@@ -866,7 +871,8 @@ void DocumentView::textChanged()
 				if (commentIndex < curIndex)
 					return;
 			}
-			if (line.contains("opcode") || line.contains("instr")) { // Don't pop menu in these cases.
+            if(line.indexOf(QRegExp("^\\s*(opcode|instr)")) >= 0) {
+            // if (line.contains("opcode") || line.contains("instr")) { // Don't pop menu in these cases.
 				return;
 			}
 			if (line.indexOf(QRegExp("\\s*\\w+\\s+\\w+\\s+")) >= 0) {
@@ -885,31 +891,35 @@ void DocumentView::textChanged()
 						}
 					}
 					foreach(QString var, vars) {
-						QAction *a = syntaxMenu->addAction(var,
-														   this, SLOT(insertAutoCompleteText())); // was: insertParameterText that does not exist any more
+                        QAction *a = syntaxMenu->addAction(var, this,
+                                                           SLOT(insertAutoCompleteText())); // was: insertParameterText that does not exist any more
 						a->setData(var);
 						if(vars.indexOf(var) == 0) {
 							syntaxMenu->setDefaultAction(a);
 						}
 					}
-					if (word.size() > 2) {
+                    if (word.size() > 2) {
+                        // qDebug() << "word: " << word << "\n";
 						// check for autcompletion from all words in text editor
 						QString wholeText = editor->toPlainText();
-						foreach (QString s, QStringList()<<"+" << "-" << "*" << "/" <<"="<<"#"<<"&"<<","
-								 << "\""<<"\'"<<"|" << "[" << "]" << "(" << ")" << "<" << ">"
-								 << "." << ";") // get rid of specific symbols
-							wholeText.replace(s," ");
-						QStringList allWords = wholeText.simplified().split(" ");
+                        wholeText.replace(QRegularExpression("[" + QRegularExpression::escape("+-*/=#&,\"\'|[]()<>.;0123456789:") + "]"), " ");
+                        QStringList allWords = wholeText.simplified().split(" ");
 						QStringList menuWords;
 						allWords.removeDuplicates();
-						allWords.replaceInStrings(QRegExp("^\\d*$"),""); // remove numbers - not good enough regexp, '.' stays
+                        // allWords.replaceInStrings(QRegExp("^\\d*$"),""); // remove numbers - not good enough regexp, '.' stays
 						allWords.removeAll("");
 						foreach(QString theWord, allWords) {
-							if (theWord.toLower().startsWith(word.toLower()) && word != theWord) {
+                            if (theWord.toLower().startsWith(wordlow) && word != theWord) {
 								menuWords << theWord;
 							}
 						}
-						foreach(QString theWord, menuWords) {
+                        foreach(QString tag, tagWords) {
+                            if(tag.toLower().startsWith(wordlow) && word != tag) {
+                                menuWords << tag;
+                            }
+
+                        }
+                        foreach(QString theWord, menuWords) {
 							QAction *a = syntaxMenu->addAction(theWord,
 															   this, SLOT(insertAutoCompleteText()));
 							a->setData(theWord);
@@ -1917,7 +1927,14 @@ void DocumentView::setParsedUDOs(QStringList udos) {
 
 MySyntaxMenu::MySyntaxMenu(QWidget * parent) :
 	QMenu(parent)
+
 {
+    this->setStyleSheet("QMenu {"
+                        "  background-color: #f8f8f8;"
+                        "  color: #5050A0 !important; "
+                        "  border-radius: 4px; "
+                        "  line-height: 95%; "
+                        "}");
 
 }
 
