@@ -44,6 +44,7 @@ QuteButton::QuteButton(QWidget *parent) : QuteWidget(parent)
 	setProperty("QCS_image", "");
 	setProperty("QCS_eventLine", "");
 	setProperty("QCS_latch", false);
+	setProperty("QCS_momentaryMidiButton", false); // used for latched button if bound to MIDI controller
     setProperty("QCS_latched", false);
     setProperty("QCS_fontsize", 10);
     
@@ -52,6 +53,8 @@ QuteButton::QuteButton(QWidget *parent) : QuteWidget(parent)
 	onIcon.addPixmap(p, QIcon::Normal, QIcon::On);
 	p.fill(QColor(Qt::black));
 	onIcon.addPixmap(p, QIcon::Normal, QIcon::Off);
+
+	m_isPlaying = false; // used for non-latched eventButton to turn the instrument off on second press
 }
 
 QuteButton::~QuteButton()
@@ -272,6 +275,7 @@ QString QuteButton::getWidgetXmlText()
 	s.writeTextElement("image", property("QCS_image").toString());
 	s.writeTextElement("eventLine", property("QCS_eventLine").toString());
 	s.writeTextElement("latch", property("QCS_latch").toString());
+	s.writeTextElement("momentaryMidiButton", property("QCS_momentaryMidiButton").toString());
 	s.writeTextElement("latched", property("QCS_latched").toString());
     s.writeTextElement("fontsize", QString::number(property("QCS_fontsize").toInt()));
 	s.writeEndElement();
@@ -302,6 +306,7 @@ void QuteButton::applyProperties()
 	setProperty("QCS_type", typeComboBox->currentText());
 	setProperty("QCS_pressedValue", valueBox->value());
 	setProperty("QCS_latch", latchCheckBox->isChecked());
+	setProperty("QCS_momentaryMidiButton", useMomentaryMidiButtonCheckBox->isChecked());
     setProperty("QCS_fontsize", fontSizeSpinBox->value());
 
 #ifdef  USE_WIDGET_MUTEX
@@ -345,8 +350,11 @@ void QuteButton::createPropertiesDialog()
 	valueBox->setDecimals(6);
 	valueBox->setRange(-9999999.0, 9999999.0);
 	valueBox->setValue(m_value);
+	valueBox->setMaximumWidth(100);
+	valueBox->setDecimals(4);
 	layout->addWidget(valueBox, 4, 3, Qt::AlignLeft|Qt::AlignVCenter);
 
+
     label = new QLabel(dialog);
 	label->setText("Text:");
 	layout->addWidget(label, 6, 0, Qt::AlignRight|Qt::AlignVCenter);
@@ -376,7 +384,7 @@ void QuteButton::createPropertiesDialog()
 
 	QPushButton *browseButton = new QPushButton(dialog);
     browseButton->setText("Browse");
-    layout->addWidget(browseButton, 8, 4, Qt::AlignLeft|Qt::AlignVCenter);
+	layout->addWidget(browseButton, 8, 4, Qt::AlignLeft|Qt::AlignVCenter);
 	connect(browseButton, SIGNAL(released()), this, SLOT(browseFile()));
 
 	label = new QLabel(dialog);
@@ -387,6 +395,21 @@ void QuteButton::createPropertiesDialog()
     layout->addWidget(line, 9,1,1,3, Qt::AlignLeft|Qt::AlignVCenter);
 	line->setMinimumWidth(320);
 	line->setText(property("QCS_eventLine").toString());
+
+	useMomentaryMidiButtonCheckBox = new QCheckBox(dialog);
+	useMomentaryMidiButtonCheckBox->setText(tr("Momentary"));
+	useMomentaryMidiButtonCheckBox->setWhatsThis(tr("Check if you use MIDI push button (momentary button) to toggle the latch - \nFirst push switches on, second off."));
+	// TODO: enabling/disabling needs some signal->slot connection
+	//useMomentaryMidiButtonCheckBox->setEnabled( latchCheckBox->isChecked() );
+	useMomentaryMidiButtonCheckBox->setChecked(property("QCS_momentaryMidiButton").toBool());
+
+	const int midiRow = layout->rowCount()-2;
+	layout->removeWidget(midiLearnButton);
+	layout->addWidget(useMomentaryMidiButtonCheckBox, midiRow, 4, Qt::AlignLeft|Qt::AlignVCenter);
+	layout->addWidget(midiLearnButton, midiRow, 5, Qt::AlignLeft|Qt::AlignVCenter);
+
+
+
 #ifdef  USE_WIDGET_MUTEX
 	widgetLock.unlock();
 #endif
@@ -406,45 +429,34 @@ void QuteButton::popUpMenu(QPoint pos)
 
 void QuteButton::setMidiValue(int value)
 {
-	double pressedVal = property("QCS_pressedValue").toDouble();
-	double newval= value == 0 ? 0 : pressedVal;
-	setValue(newval);
-	QString type = property("QCS_type").toString();
-    // if event type, start/stop event on MIDI button press/release
-    if (m_valueChanged && (type == "event" || type == "pictevent") ) {
-		QString eventLine = property("QCS_eventLine").toString();
-		if (newval==1) {
-			emit(queueEventSignal(eventLine));
-		}
-		// on release check if event was with indefinite duration, then turn it off
-		if (newval==0 && eventLine.size() > 0) {
-			QStringList lineElements = eventLine.split(QRegExp("\\s"),QString::SkipEmptyParts);
-			if (lineElements.size() > 0 && lineElements[0] == "i") {
-                // Remove first element if it is "i"
-                lineElements.removeAt(0);
+	double pressedValue = property("QCS_pressedValue").toDouble();
+	double newValue = 0;
+
+	bool isLatch = property("QCS_latch").toBool();
+	bool useMomentaryMidiButton = property("QCS_momentaryMidiButton").toBool();
+
+	if (isLatch && useMomentaryMidiButton) {
+			if (value >0 ) {
+				if (m_isPlaying) {
+					//qDebug() << "Toggle playing off from MIDI";
+					newValue = 0;
+				} else {
+					newValue = pressedValue;
+					//qDebug() << "Toggle playing on from MIDI";
+				}
+
+
+			} else {
+				qDebug() << "Ignore button release of momentary button";
+				return;
 			}
-			else if (lineElements.size() > 0 && lineElements[0][0] == 'i') {
-				lineElements[0] = lineElements[0].mid(1); // Remove "i" character
-			}
-			//bool test = property("QCS_latch").toBool();
-            if (lineElements.size() > 2 &&
-                    lineElements[2].toDouble() < 0 &&
-                    property("QCS_latch").toBool() ) {
-                // If duration is negative, turn it off. But not when it is not a latched button!
-				lineElements[0].prepend("-");
-				lineElements.prepend("i");
-				emit(queueEventSignal(lineElements.join(" ")));
-			}
-		}
-	} else if ((type == "value" || type == "pictvalue") && newval == pressedVal) {
-        QStringList reservedChannels = QStringList() << "_Play" <<"_Stop" << "_Pause"
-                                                     << "_Render" << "_Browse1"
-                                                     << "_Browse2" << "_MBrowse";
-        if (reservedChannels.contains(m_channel))
-            buttonReleased();
+
+	} else {
+		newValue = (value > 0) ? pressedValue : 0;
 	}
-	QPair<QString, double> channelValue(m_channel, newval);
-	emit newValue(channelValue);
+
+	setValue(newValue);
+	performAction();
 
 }
 
@@ -471,7 +483,7 @@ void QuteButton::refreshWidget()
 void QuteButton::applyInternalProperties()
 {
 	QuteWidget::applyInternalProperties();
-    qDebug() << "QuteButton::applyInternalProperties";
+    //qDebug() << "QuteButton::applyInternalProperties";
     m_value = property("QCS_pressedValue").toDouble();
 	//  m_value2 = property("QCS_value2").toDouble();
 	m_stringValue = property("QCS_stringvalue").toString();
@@ -514,11 +526,6 @@ void QuteButton::applyInternalProperties()
         qDebug() << "Warning! QuteButton::applyInternalProperties() unrecognized type:"
                  << type;
 	}
-
-
-    // Why is this here?
-    // QString colorstr = property("QCS_color").value<QColor>().name();
-    // m_widget->setStyleSheet("QPushButton { border-color:" + colorstr + "; }");
 }
 
 
@@ -526,43 +533,40 @@ void QuteButton::performAction() {
     QString type = property("QCS_type").toString();
     QString eventLine = property("QCS_eventLine").toString();
     QString name = m_channel;
+	bool isLatch = property("QCS_latch").toBool();
+	bool useMomentaryMidiButton = property("QCS_momentaryMidiButton").toBool();
 
-    if (type == "event" || type == "pictevent") {
-        if (property("QCS_latch").toBool() && eventLine.size() > 0) {
-            QStringList lineElements = eventLine.split(QRegExp("\\s"),QString::SkipEmptyParts);
-            if (lineElements.size() > 0 && lineElements[0] == "i") {
-                lineElements.removeAt(0); // Remove first element if it is "i"
-            }
-            else if (lineElements.size() > 0 && lineElements[0][0] == 'i') {
-                lineElements[0] = lineElements[0].mid(1); // Remove "i" character
-            }
-            // If duration is negative, use button to turn note on and off
-            if (lineElements.size() > 2 && lineElements[2].toDouble() < 0) {
-                if (m_currentValue == 0) { // Button has turned off. Turn off instrument
-                    if ( lineElements[0].startsWith("\"") || lineElements[0].startsWith("\'")  ) {
-                        //qDebug()<<"Stopping named instrument: " << lineElements[0];
-                        lineElements[0].insert(1,"-");
-                    }
-                    else {
-                        // NB! what if string? find out instrument number
-                        lineElements[0].prepend("-");
-                    }
-                    lineElements.prepend("i");
-					setValue(0); // was 1
-                    emit(queueEventSignal(lineElements.join(" ")));
-                }
-                else { // Button has turned on. Turn on instrument
-					setValue(1); // was 0
-                    emit(queueEventSignal(eventLine));
-                }
-            }
-            else {
-                emit(queueEventSignal(eventLine));
-            }
-        }
-        else {
-            emit(queueEventSignal(eventLine));
-        }
+	if (type.contains("event") && !eventLine.isEmpty()) {
+		if ( hasIndefiniteDuration() ) {		
+			if ( m_currentValue == 0  /*|| (!useMomentaryMidiButton && m_isPlaying)*/ ) { // turn off
+				QStringList lineElements = eventLine.split(QRegExp("\\s"),QString::SkipEmptyParts);
+				if (lineElements.size() > 0 && lineElements[0] == "i") {
+					lineElements.removeAt(0); // Remove first element if it is "i"
+				}
+				else if (lineElements.size() > 0 && lineElements[0][0] == 'i') {
+					lineElements[0] = lineElements[0].mid(1); // Remove "i" character
+				}
+				if ( lineElements[0].startsWith("\"") || lineElements[0].startsWith("\'")  ) {
+					//qDebug()<<"Stopping named instrument: " << lineElements[0];
+					lineElements[0].insert(1,"-");
+				}
+				else {
+					lineElements[0].prepend("-");
+				}
+				lineElements.prepend("i");
+				setValue(0);
+				m_isPlaying = false;
+				emit(queueEventSignal(lineElements.join(" ")));
+			} else {
+				setValue( property("QCS_pressedValue").toDouble()  ); // was 1
+				m_isPlaying = true;
+				emit(queueEventSignal(eventLine));
+			}
+		} else { // if not negative p3 then just fire the event
+			if (!isLatch && m_currentValue>0) { //do not fire the event if latched && m_value==0 && is positive p3
+				emit(queueEventSignal(eventLine));
+			}
+		}
     }
     else if (type == "value" || type == "pictvalue") {
         if(!name.startsWith("_")) {
@@ -601,7 +605,26 @@ void QuteButton::performAction() {
                         "be an error in a next release";
             emit newValue(QPair<QString, double>(name, m_currentValue));
         }
-    }
+	}
+}
+
+bool QuteButton::hasIndefiniteDuration()
+{
+	QString eventLine = property("QCS_eventLine").toString();
+	if ( !eventLine.isEmpty()) {
+		QStringList lineElements = eventLine.split(QRegExp("\\s"),QString::SkipEmptyParts);
+		if (lineElements.size() > 0 && lineElements[0] == "i") {
+			lineElements.removeAt(0); // Remove first element if it is "i"
+		}
+		else if (lineElements.size() > 0 && lineElements[0][0] == 'i') {
+			lineElements[0] = lineElements[0].mid(1); // Remove "i" character
+		}
+		// If duration is negative, use button to turn note on and off
+		if (lineElements.size() > 2 && lineElements[2].toDouble() < 0) {
+			return true;
+		}
+	}
+	return false;
 }
 
 void QuteButton::buttonPressed()
@@ -620,6 +643,8 @@ void QuteButton::buttonPressed()
 		m_currentValue = m_value;
     }
 
+
+
 #ifdef  USE_WIDGET_MUTEX
 	widgetLock.unlock();
 #endif
@@ -636,9 +661,15 @@ void QuteButton::buttonReleased()
         performAction();
         return;
     }
-    if (!property("QCS_latch").toBool()) {
+	bool isLatch = property("QCS_latch").toBool();
+
+	if (!isLatch ) {
         m_currentValue = 0;
-    }
+		if (  property("QCS_type").toString().contains("event") && hasIndefiniteDuration() ) {
+			performAction(); // to stop the playing instrument
+		}
+	}
+
 
 #ifdef  USE_WIDGET_MUTEX
 	widgetLock.unlock();
