@@ -23,22 +23,20 @@
 #ifndef QUTECSOUND_H
 #define QUTECSOUND_H
 
-#ifdef USE_QT5
 #include <QtWidgets>
 #ifdef USE_QT_GT_53
 #include <QQuickWidget>
 #include <QQuickItem>
 #endif
-#else
-#include <QtGui>
-#include <QDesktopServices>
-#endif
+
 #include <QLocalServer>
 #include <QLocalSocket>
 
 #include "types.h"
 #include "configlists.h"
 
+#include "risset.h"
+
 #ifdef MACOSX_PRE_SNOW
 // Needed to be able to grab menus back from FLTK
 #include <Carbon/Carbon.h>
@@ -102,7 +100,8 @@ public:
 	void setPresetsText(QString text, int index = -1);
 	void setOptionsText(QString text, int index = -1);
 	int getDocument(QString name = ""); // Returns document index. -1 if not current open
-	QString getSelectedText(int index = -1, int section = 0);
+    QString getTheme();
+    QString getSelectedText(int index = -1, int section = 0);
 	QString getCsd(int index);
 	QString getFullText(int index);
 	QString getOrc(int index);
@@ -146,6 +145,7 @@ public:
 	// Engine
 	CsoundEngine *getEngine(int index = -1);
 	OpEntryParser *m_opcodeTree;
+    Risset *risset;
 	void stkCheck();
 	// localServer
 	bool startServer();
@@ -168,6 +168,7 @@ public slots:
 	void perfEnded();
 	void render();
 	void record(bool);
+	void record(bool, int index);
 	void sendEvent(QString eventLine, double delay = 0);
 	void sendEvent(int index, QString line, double delay = 0);
 	void changePage(int index);
@@ -224,6 +225,7 @@ private slots:
 	void showUtilities(bool);
 	void getToIn();
 	void inToGet();
+	void insertMidiControlInstrument();
 	void updateCsladspaText();
 	void updateCabbageText();
     void saveWidgetsToQml();
@@ -232,6 +234,7 @@ private slots:
 	void openExternalPlayer();
 	void setEditorFocus();
 	void setHelpEntry();
+    void helpForEntry(QString text, bool external=false);
     void setFullScreen(bool full);
     void checkFullScreen();
     void setEditorFullScreen(bool full);
@@ -255,11 +258,11 @@ private slots:
 	void openFLOSSManual();
 	void openQuickRef();
 	void openOnlineDocumentation();
+	void showManualOnline();
 	void resetPreferences();
 	void reportBug();
-	void requestFeature();
-	void chat();
-	void openShortcutDialog();
+    void reportCsoundBug();
+    void openShortcutDialog();
 	void downloadManual();
 	void about();
 	void donate();
@@ -291,6 +294,7 @@ private slots:
     void ambiguosShortcut();
     void testAudioSetup();
     void checkSyntaxMenuAction();
+    void tabMoved(int to, int from);
 
     DocumentPage *getCurrentDocumentPage() {
         if(curPage >= documentPages.size())
@@ -315,6 +319,8 @@ private:
 	void connectActions();
 	QString getExamplePath(QString dir);
 	void createMenus();
+	void fillExampleMenu();
+    void fillExampleSubmenu(QDir dir, QMenu *m, int depth);
 	void fillFileMenu();
 	void fillFavoriteMenu();
 	void fillFavoriteSubMenu(QDir dir, QMenu *m, int depth);
@@ -328,8 +334,8 @@ private:
 	void writeSettings(QStringList openFiles=QStringList(), int lastIndex = 0);
 	void clearSettings();
     void setToolbarIconSize(int size);
-	int execute(QString executable, QString options);
-	//    bool saveCurrent();
+    int execute(QString executable, QString options);
+    //    bool saveCurrent();
 	bool makeNewPage(QString fileName, QString text);
 	bool loadCompanionFile(const QString &fileName);
 	void setCurrentFile(const QString &fileName);
@@ -413,6 +419,7 @@ private:
 	QAction *scratchPadCsdModeAct;
 	QAction *getToInAct;
 	QAction *inToGetAct;
+	QAction *midiControlAct;
 	QAction *csladspaAct;
 	QAction *cabbageAct;
     QAction *qmlAct;
@@ -436,6 +443,7 @@ private:
 	QAction *showHelpAct;
     QAction *raiseHelpAct;
 	QAction *showManualAct;
+	QAction *showManualOnlineAct;
 	QAction *downloadManualAct;
 	QAction *showGenAct;
 	QAction *showOverviewAct;
@@ -495,9 +503,8 @@ private:
 	//    QAction *aboutQtAct;
 	QAction *resetPreferencesAct;
 	QAction *reportBugAct;
-	QAction *requestFeatureAct;
-	QAction *chatAct;
-	QAction *lineNumbersAct;
+    QAction *reportCsoundBugAct;
+    QAction *lineNumbersAct;
 	QAction *parameterModeAct;
 //	QAction *showParametersAct;
     QSignalMapper *focusMapper;
@@ -525,6 +532,8 @@ private:
 #endif
     QByteArray m_preFullScreenState;
     QString m_fullScreenComponent;
+    QDir m_rissetDataPath;
+    bool isDarkPalette;
 };
 
 class FileOpenEater : public QObject
@@ -536,7 +545,7 @@ public:
 		m_mw = mainWindow;
 		while (!fileEventQueue.isEmpty()) {
 			QString fileName = fileEventQueue.takeFirst();
-			qDebug() << "FileOpenEater::setMainWindow  opening file " << fileName << endl;
+            qDebug() << "FileOpenEater::setMainWindow  opening file" << fileName << Qt::endl;
 			m_mw->loadFileFromSystem(fileName);
 		}
 	}
